@@ -1,8 +1,7 @@
 """ pydatastream main module
 
-    (c) Vladimir Filimonov, 2013 - 2020
+    (c) Vladimir Filimonov, 2013 - 2021
 """
-# pylint: disable=C0103,R0902,R0904,R0913,C0330
 import warnings
 import json
 import math
@@ -165,8 +164,8 @@ class Datastream():
 
         try:
             response = self.last_request['response'] = json.loads(self.last_request['response'])
-        except json.JSONDecodeError:
-            raise DatastreamException('Server response could not be parsed')
+        except json.JSONDecodeError as e:
+            raise DatastreamException('Server response could not be parsed') from e
 
         if 'Code' in response:
             code = response['Code']
@@ -185,14 +184,20 @@ class Datastream():
             return
         data = {"UserName": username, "Password": password}
         self._token = dict(self._api_post('GetToken', data))
-        self._token['TokenExpiry'] = _parse_dates(self._token['TokenExpiry'])
+        self._token['TokenExpiry'] = _parse_dates(self._token['TokenExpiry']).tz_localize('UTC')
+
+        # Token is invalidated 15 minutes before exporation time
+        # Note: According to https://github.com/vfilimonov/pydatastream/issues/27
+        #       tokens do not always respect the (as of now 24 hours) expiry time
+        #       So for this reason I limit the token life at 6 hours.
+        self._token['RenewTokenAt'] = min(self._token['TokenExpiry'] - pd.Timedelta('15m'),
+                                          pd.Timestamp.utcnow() + pd.Timedelta('6H'))
 
     @property
     def _token_is_expired(self):
         if self._token is None:
             return True
-        # We invalidate token 15 minutes before expiration time
-        if self._token['TokenExpiry'] < pd.Timestamp('now') - pd.Timedelta('15m'):
+        if pd.Timestamp.utcnow() > self._token['RenewTokenAt']:
             return True
         return False
 
@@ -256,7 +261,7 @@ class Datastream():
 
         # Instruments
         if isinstance(ticker, str):
-            ticker = ticker
+            # ticker = ticker  ## Nothing to change
             is_list = None
         elif hasattr(ticker, '__len__'):
             ticker = ','.join(ticker)
@@ -392,7 +397,8 @@ class Datastream():
 
     ###########################################################################
     def fetch(self, tickers, fields=None, date_from=None, date_to=None,
-              freq=None, static=False, IsExpression=None, return_metadata=False):
+              freq=None, static=False, IsExpression=None, return_metadata=False,
+              always_multiindex=False):
         """Fetch the data from Datastream for a set of tickers and parse results.
 
            tickers - ticker or symbol, or list of symbols
@@ -402,6 +408,12 @@ class Datastream():
            static  - True for static (snapshot) requests
            IsExpression - if True, it will explicitly assume that list of tickers
                           contain expressions. Otherwise it will try to infer it.
+           return_metadata - if True, then tuple (data, metadata) will be returned,
+                             otherwise only data
+           always_multiindex - if True, then even for 1 ticker requested, resulting
+                               dataframe will have multiindex (ticker, date).
+                               If False (default), then request of single ticker will
+                               result in a dataframe indexed by date only.
 
            Notes: - several fields should be passed as a list, and not as a
                     comma-separated string!
@@ -411,7 +423,9 @@ class Datastream():
 
            Result format depends on the number of requested tickers and fields:
              - 1 ticker         - DataFrame with fields in column names
-             - many tickers     - DataFrame with fields in column names and
+                                  (in order to keep multiindex even for single
+                                  ticker, set `always_multiindex` to True)
+             - several tickers  - DataFrame with fields in column names and
                                   MultiIndex (ticker, date)
              - static request   - DataFrame indexed by tickers and with fields
                                   in column names
@@ -445,7 +459,8 @@ class Datastream():
             data = data.reset_index(level=1, drop=True)
         elif len(data.index.levels[0]) == 1:
             # Only one ticker - drop tickers from MultiIndex
-            data = data.reset_index(level=0, drop=True)
+            if not always_multiindex:
+                data = data.reset_index(level=0, drop=True)
 
         return (data, meta) if return_metadata else data
 
@@ -531,7 +546,7 @@ class Datastream():
         if isinstance(symbols, (list, pd.Series)):
             try:
                 res = res.loc[symbols]
-            except:
+            except KeyError:
                 pass  # OK, we don't keep the order if not possible
         return res
 
